#![no_std]
#![no_main]

use panic_halt as _;

use rp235x_hal as hal;


use embedded_hal::delay::DelayNs;
use embedded_hal::digital::InputPin;
use embedded_hal::digital::OutputPin;
use embedded_hal_0_2::adc::OneShot;

use usb_device::{class_prelude::*, prelude::*};
use usbd_hid::hid_class::HIDClass;

mod hid; use hid::*;

#[link_section = ".start_block"]
#[used]
pub static IMAGE_DEF :hal::block::ImageDef = hal::block::ImageDef::secure_exe();

const XOSC_CRYSTAL_FREQ :u32 = 12_000_000;

#[hal::entry]
fn main() -> ! {
    /*
    ** Hardware Resources
    */
    
    let mut pac = hal::pac::Peripherals::take().unwrap();
    let mut watchdog = hal::Watchdog::new(pac.WATCHDOG);

    let clocks = hal::clocks::init_clocks_and_plls(
        XOSC_CRYSTAL_FREQ,
        pac.XOSC,
        pac.CLOCKS,
        pac.PLL_SYS,
        pac.PLL_USB,
        &mut pac.RESETS,
        &mut watchdog,
    ).unwrap();

    let mut timer = hal::Timer::new_timer0(pac.TIMER0, &mut pac.RESETS, &clocks);

    let sio = hal::Sio::new(pac.SIO);

    let mut adc = hal::adc::Adc::new(pac.ADC, &mut pac.RESETS);

    let pins = hal::gpio::Pins::new(
        pac.IO_BANK0,
        pac.PADS_BANK0,
        sio.gpio_bank0,
        &mut pac.RESETS,
    );

    // Acquire GPIO pin handles.
    let mut light = pins.gpio0.into_push_pull_output();
    let mut led = pins.gpio25.into_push_pull_output();

    let mut button1 = pins.gpio1.into_pull_up_input();
    let mut button2 = pins.gpio4.into_pull_up_input();
    let mut button3 = pins.gpio6.into_pull_up_input();
    let mut button4 = pins.gpio21.into_pull_up_input();
    let mut button5 = pins.gpio19.into_pull_up_input();
    let mut button6 = pins.gpio18.into_pull_up_input();
    let mut button7 = pins.gpio17.into_pull_up_input();
    let mut button8 = pins.gpio16.into_pull_up_input();

    let mut adc2 = hal::adc::AdcPin::new(pins.gpio28).unwrap();


    /*
    ** USB HID Resources
    */

    // Select device report.
    type Report = report::Mouse;
    
    // Prepare USB, HID controllers.
    let usb_alloc = UsbBusAllocator::new(hal::usb::UsbBus::new(
        pac.USB,
        pac.USB_DPRAM,
        clocks.usb_clock,
        true,
        &mut pac.RESETS,
    ));
    let mut hid = HIDClass::new(&usb_alloc, Report::descriptor(), 10);
    let mut usb_dev = UsbDeviceBuilder::new(&usb_alloc, UsbVidPid(0x9727, 0x0024))
    .build();


    /*
    ** Program Resources
    */

    let mut report = Report::default();
    let mut last_report = report;
    let mut report_buffer = [0u8; 64];

    /*
    ** Main Program
    */

    led.set_high().unwrap();
    loop {
        timer.delay_ms(10);

        // Generate report from component states.
        let pot_x :u16 = adc.read(&mut adc2).unwrap();
        //let pot_x = 0;
        report.x = (((pot_x as i16) - 2048) / 16) as i8;
        report.buttons = button1.is_low().unwrap_or_default() as u8
        | ((button2.is_low().unwrap_or_default() as u8) << 1)
        | ((button3.is_low().unwrap_or_default() as u8) << 2)
        | ((button4.is_low().unwrap_or_default() as u8) << 3)
        | ((button5.is_low().unwrap_or_default() as u8) << 4)
        | ((button6.is_low().unwrap_or_default() as u8) << 5)
        | ((button7.is_low().unwrap_or_default() as u8) << 6)
        | ((button8.is_low().unwrap_or_default() as u8) << 7);

        // Buffer USB event on change.
        if report != last_report {
            
            // Set LED if button is pressed.
            if report.buttons != 0 {
                light.set_high().unwrap();
                led.set_low().unwrap();
            } else {
                light.set_low().unwrap();
                //led.set_high().unwrap();
            }

            // Push report to USB.
            let len = report.report(1, &mut report_buffer);
            hid.push_raw_input(&report_buffer[..len]).ok();

            last_report = report;
        }

        // Dispatch USB events.
        if usb_dev.poll(&mut [&mut hid]) { }
    }
}
